@@ -9,10 +9,11 @@ from optparse import OptionParser, OptionGroup
 import os
 import pkg_resources
 import re
+import requests
 import sys
 import time
 import urllib
-import urllib2
+from urlparse import urlparse
 
 import mozinfo
 
@@ -23,18 +24,21 @@ import progressbar
 
 version = pkg_resources.require("mozdownload")[0].version
 
-__doc__= """
-Module to handle downloads for different types of ftp.mozilla.org hosted 
+__doc__ = """
+Module to handle downloads for different types of ftp.mozilla.org hosted
 applications.
 
 mozdownload version: %(version)s
-""" % {'version' : version}
+""" % {'version': version}
 
 APPLICATIONS = ['b2g', 'firefox', 'thunderbird']
 
 # Base URL for the path to all builds
 BASE_URL = 'https://ftp.mozilla.org/pub/mozilla.org'
 
+# Chunk size when downloading a file
+CHUNK_SIZE = 16 * 1024
+
 PLATFORM_FRAGMENTS = {'linux': 'linux-i686',
                       'linux64': 'linux-x86_64',
                       'mac': 'mac',
@@ -49,14 +53,21 @@ DEFAULT_FILE_EXTENSIONS = {'linux': 'tar.bz2',
                            'win32': 'exe',
                            'win64': 'exe'}
 
-class NotFoundException(Exception):
+
+class NotFoundError(Exception):
     """Exception for a resource not being found (e.g. no logs)"""
     def __init__(self, message, location):
         self.location = location
         Exception.__init__(self, ': '.join([message, location]))
 
 
-class TimeoutException(Exception):
+class NotImplementedError(Exception):
+    """Exception for a feature which is not implemented yet"""
+    def __init__(self, message):
+        Exception.__init__(self, message)
+
+
+class TimeoutError(Exception):
     """Exception for a download exceeding the allocated timeout"""
     def __init__(self):
         self.message = 'The download exceeded the allocated timeout'
@@ -68,8 +79,8 @@ class Scraper(object):
 
     def __init__(self, directory, version, platform=None,
                  application='firefox', locale='en-US', extension=None,
-                 authentication=None, retry_attempts=0, retry_delay=10,
-                 timeout=180):
+                 authentication=None, retry_attempts=0, retry_delay=10.,
+                 timeout=None):
 
         # Private properties for caching
         self._target = None
@@ -83,39 +94,44 @@ class Scraper(object):
         self.authentication = authentication
         self.retry_attempts = retry_attempts
         self.retry_delay = retry_delay
-        self.timeout = timeout
+        self.timeout_download = timeout
+        self.timeout_network = 60.
 
         # build the base URL
         self.application = application
         self.base_url = '/'.join([BASE_URL, self.application])
 
-        self.opener = None
-        if self.authentication \
-           and self.authentication['username'] \
-           and self.authentication['password']:
-            password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
-            password_mgr.add_password(None,
-                                      self.base_url,
-                                      self.authentication['username'],
-                                      self.authentication['password'])
-            handler = urllib2.HTTPBasicAuthHandler(password_mgr)
-            opener = urllib2.build_opener(urllib2.HTTPHandler, handler)
-            urllib2.install_opener(opener)
-            self.opener = opener
+        attempt = 0
+        while True:
+            attempt += 1
+            try:
+                self.get_build_info()
+                break
+            except (NotFoundError, requests.exceptions.RequestException), e:
+                if self.retry_attempts > 0:
+                    # Print only if multiple attempts are requested
+                    print "Build not found: '%s'" % e.message
+                    print 'Will retry in %s seconds...' % self.retry_delay
+                    time.sleep(self.retry_delay)
+                    print "Retrying... (attempt %s)" % attempt
+                if attempt >= self.retry_attempts:
+                    raise
 
     @property
     def binary(self):
         """Return the name of the build"""
-        
+
         attempt = 0
 
         while self._binary is None:
             attempt += 1
             try:
                 # Retrieve all entries from the remote virtual folder
-                parser = DirectoryParser(self.path)
+                parser = DirectoryParser(self.path,
+                                         authentication=self.authentication,
+                                         timeout=self.timeout_network)
                 if not parser.entries:
-                    raise NotFoundException('No entries found', self.path)
+                    raise NotFoundError('No entries found', self.path)
 
                 # Download the first matched directory entry
                 pattern = re.compile(self.binary_regex, re.IGNORECASE)
@@ -127,16 +143,17 @@ class Scraper(object):
                         # No match, continue with next entry
                         continue
                 else:
-                    raise NotFoundException("Binary not found in folder",
-                                            self.path)
-            except (NotFoundException, urllib2.HTTPError):
+                    raise NotFoundError("Binary not found in folder",
+                                        self.path)
+            except (NotFoundError, requests.exceptions.RequestException), e:
                 if self.retry_attempts > 0:
                     # Print only if multiple attempts are requested
-                    print "Binary not found! Retrying... (attempt %s)" % attempt
+                    print "Build not found: '%s'" % e.message
+                    print 'Will retry in %s seconds...' % self.retry_delay
+                    time.sleep(self.retry_delay)
+                    print "Retrying... (attempt %s)" % attempt
                 if attempt >= self.retry_attempts:
-                    raise NotFoundException("Binary not found in folder",
-                                            self.path)
-                time.sleep(self.retry_delay)
+                    raise
 
         return self._binary
 
@@ -146,34 +163,29 @@ class Scraper(object):
 
         raise NotImplementedError(sys._getframe(0).f_code.co_name)
 
-
     @property
     def final_url(self):
         """Return the final URL of the build"""
 
         return '/'.join([self.path, self.binary])
 
-
     @property
     def path(self):
         """Return the path to the build"""
 
         return '/'.join([self.base_url, self.path_regex])
 
-
     @property
     def path_regex(self):
         """Return the regex for the path to the build"""
 
         raise NotImplementedError(sys._getframe(0).f_code.co_name)
 
-
     @property
     def platform_regex(self):
         """Return the platform fragment of the URL"""
 
-        return PLATFORM_FRAGMENTS[self.platform];
-
+        return PLATFORM_FRAGMENTS[self.platform]
 
     @property
     def target(self):
@@ -184,23 +196,25 @@ class Scraper(object):
                                         self.build_filename(self.binary))
         return self._target
 
+    def get_build_info(self):
+        """Returns additional build information in subclasses if necessary"""
+        pass
 
     def build_filename(self, binary):
         """Return the proposed filename with extension for the binary"""
 
         raise NotImplementedError(sys._getframe(0).f_code.co_name)
 
-
     def detect_platform(self):
         """Detect the current platform"""
 
         # For Mac and Linux 32bit we do not need the bits appended
-        if mozinfo.os == 'mac' or (mozinfo.os == 'linux' and mozinfo.bits == 32):
+        if mozinfo.os == 'mac' or \
+                (mozinfo.os == 'linux' and mozinfo.bits == 32):
             return mozinfo.os
         else:
             return "%s%d" % (mozinfo.os, mozinfo.bits)
 
-
     def download(self):
         """Download the specified file"""
 
@@ -210,7 +224,8 @@ class Scraper(object):
             if hasattr(td, 'total_seconds'):
                 return td.total_seconds()
             else:
-                return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6
+                return (td.microseconds +
+                        (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6
 
         attempt = 0
 
@@ -229,39 +244,58 @@ class Scraper(object):
             attempt += 1
             try:
                 start_time = datetime.now()
-                r = urllib2.urlopen(self.final_url)
-                total_size = int(r.info().getheader('Content-length').strip())
-                CHUNK = 16 * 1024
+
+                # Enable streaming mode so we can download content in chunks
+                r = requests.get(self.final_url, stream=True,
+                                 auth=self.authentication)
+                r.raise_for_status()
+
                 # ValueError: Value out of range if only total_size given
-                max_value = ((total_size / CHUNK) + 1) * CHUNK
+                total_size = int(r.headers.get('Content-length').strip())
+                max_value = ((total_size / CHUNK_SIZE) + 1) * CHUNK_SIZE
                 bytes_downloaded = 0
                 widgets = [progressbar.Percentage(), ' ', progressbar.Bar(),
-                           ' ', progressbar.ETA(), ' ', progressbar.FileTransferSpeed()]
-                pbar = progressbar.ProgressBar(widgets=widgets, maxval=max_value).start()
+                           ' ', progressbar.ETA(), ' ',
+                           progressbar.FileTransferSpeed()]
+                pbar = progressbar.ProgressBar(widgets=widgets,
+                                               maxval=max_value).start()
 
                 with open(tmp_file, 'wb') as f:
-                    for chunk in iter(lambda: r.read(CHUNK), ''):
+                    for chunk in iter(lambda: r.raw.read(CHUNK_SIZE), ''):
                         f.write(chunk)
-                        bytes_downloaded += CHUNK
+                        bytes_downloaded += CHUNK_SIZE
                         pbar.update(bytes_downloaded)
 
                         t1 = total_seconds(datetime.now() - start_time)
-                        if t1 >= self.timeout:
-                            raise TimeoutException
+                        if self.timeout_download and \
+                                t1 >= self.timeout_download:
+                            raise TimeoutError
                 pbar.finish()
                 break
-            except (urllib2.HTTPError, urllib2.URLError, TimeoutException):
+            except (requests.exceptions.RequestException, TimeoutError), e:
                 if tmp_file and os.path.isfile(tmp_file):
                     os.remove(tmp_file)
                 if self.retry_attempts > 0:
                     # Print only if multiple attempts are requested
-                    print '\nDownload failed! Retrying... (attempt %s)' % attempt
+                    print 'Download failed: "%s"' % e.message
+                    print 'Will retry in %s seconds...' % self.retry_delay
+                    time.sleep(self.retry_delay)
+                    print "Retrying... (attempt %s)" % attempt
                 if attempt >= self.retry_attempts:
                     raise
                 time.sleep(self.retry_delay)
 
         os.rename(tmp_file, self.target)
 
+    def show_matching_builds(self, builds):
+        """Output the matching builds"""
+        print 'Found %s build%s: %s' % (
+            len(builds),
+            len(builds) > 1 and 's' or '',
+            len(builds) > 10 and
+            ' ... '.join([', '.join(builds[:5]), ', '.join(builds[-5:])]) or
+            ', '.join(builds))
+
 
 class DailyScraper(Scraper):
     """Class to download a daily build from the Mozilla server"""
@@ -269,28 +303,36 @@ class DailyScraper(Scraper):
     def __init__(self, branch='mozilla-central', build_id=None, date=None,
                  build_number=None, *args, **kwargs):
 
-        Scraper.__init__(self, *args, **kwargs)
         self.branch = branch
+        self.build_id = build_id
+        self.date = date
+        self.build_number = build_number
+
+        Scraper.__init__(self, *args, **kwargs)
+
+    def get_build_info(self):
+        """Defines additional build information"""
 
         # Internally we access builds via index
-        if build_number is not None:
-            self.build_index = int(build_number) - 1
+        if self.build_number is not None:
+            self.build_index = int(self.build_number) - 1
         else:
             self.build_index = None
 
-        if build_id:
-            # A build id has been specified. Split up its components so the date
-            # and time can be extracted: '20111212042025' -> '2011-12-12 04:20:25'
-            self.date = datetime.strptime(build_id, '%Y%m%d%H%M%S')
-            self.builds, self.build_index = self.get_build_info_for_date(self.date,
-                                                                         has_time=True)
+        if self.build_id:
+            # A build id has been specified. Split up its components so the
+            # date and time can be extracted:
+            # '20111212042025' -> '2011-12-12 04:20:25'
+            self.date = datetime.strptime(self.build_id, '%Y%m%d%H%M%S')
+            self.builds, self.build_index = self.get_build_info_for_date(
+                self.date, has_time=True)
 
-        elif date:
+        elif self.date:
             # A date (without time) has been specified. Use its value and the
             # build index to find the requested build for that day.
-            self.date = datetime.strptime(date, '%Y-%m-%d')
-            self.builds, self.build_index = self.get_build_info_for_date(self.date,
-                                                                         build_index=self.build_index)
+            self.date = datetime.strptime(self.date, '%Y-%m-%d')
+            self.builds, self.build_index = self.get_build_info_for_date(
+                self.date, build_index=self.build_index)
 
         else:
             # If no build id nor date have been specified the lastest available
@@ -299,46 +341,75 @@ class DailyScraper(Scraper):
             url = '%s/nightly/latest-%s/' % (self.base_url, self.branch)
 
             print 'Retrieving the build status file from %s' % url
-            parser = DirectoryParser(url)
+            parser = DirectoryParser(url, authentication=self.authentication,
+                                     timeout=self.timeout_network)
             parser.entries = parser.filter(r'.*%s\.txt' % self.platform_regex)
             if not parser.entries:
-                message = 'Status file for %s build cannot be found' % self.platform_regex
-                raise NotFoundException(message, url)
+                message = 'Status file for %s build cannot be found' % \
+                    self.platform_regex
+                raise NotFoundError(message, url)
 
-            # Read status file for the platform, retrieve build id, and convert to a date
-            status_file = url + parser.entries[-1]
-            f = urllib.urlopen(status_file)
-            self.date = datetime.strptime(f.readline().strip(), '%Y%m%d%H%M%S')
-            self.builds, self.build_index = self.get_build_info_for_date(self.date,
-                                                                         has_time=True)
+            # Read status file for the platform, retrieve build id,
+            # and convert to a date
+            headers = {'Cache-Control': 'max-age=0'}
+            r = requests.get(url + parser.entries[-1],
+                             auth=self.authentication, headers=headers)
+            r.raise_for_status()
+
+            self.date = datetime.strptime(r.text.split('\n')[0],
+                                          '%Y%m%d%H%M%S')
+            self.builds, self.build_index = self.get_build_info_for_date(
+                self.date, has_time=True)
+
+    def is_build_dir(self, dir):
+        """Return whether or not the given dir contains a build."""
+
+        url = '/'.join([self.base_url, self.monthly_build_list_regex, dir])
+        parser = DirectoryParser(url, authentication=self.authentication,
+                                 timeout=self.timeout_network)
+
+        pattern = re.compile(self.binary_regex, re.IGNORECASE)
+        for entry in parser.entries:
+            try:
+                pattern.match(entry).group()
+                return True
+            except:
+                # No match, continue with next entry
+                continue
+        return False
 
 
     def get_build_info_for_date(self, date, has_time=False, build_index=None):
         url = '/'.join([self.base_url, self.monthly_build_list_regex])
 
         print 'Retrieving list of builds from %s' % url
-        parser = DirectoryParser(url)
+        parser = DirectoryParser(url, authentication=self.authentication,
+                                 timeout=self.timeout_network)
         regex = r'%(DATE)s-(\d+-)+%(BRANCH)s%(L10N)s$' % {
-                    'DATE': date.strftime('%Y-%m-%d'),
-                    'BRANCH': self.branch,
-                    'L10N': '' if self.locale == 'en-US' else '-l10n'}
+            'DATE': date.strftime('%Y-%m-%d'),
+            'BRANCH': self.branch,
+            'L10N': '' if self.locale == 'en-US' else '(-l10n)?'}
         parser.entries = parser.filter(regex)
-        if not parser.entries:
-            message = 'Folder for builds on %s has not been found' % self.date.strftime('%Y-%m-%d')
-            raise NotFoundException(message, url)
+        parser.entries = parser.filter(self.is_build_dir)
 
         if has_time:
-            # If a time is included in the date, use it to determine the build's index
+            # If a time is included in the date, use it to determine the
+            # build's index
             regex = r'.*%s.*' % date.strftime('%H-%M-%S')
-            build_index = parser.entries.index(parser.filter(regex)[0])
-        else:
-            # If no index has been given, set it to the last build of the day.
-            if build_index is None:
-                build_index = len(parser.entries) - 1
+            parser.entries = parser.filter(regex)
+
+        if not parser.entries:
+            message = 'Folder for builds on %s has not been found' % \
+                self.date.strftime('%Y-%m-%d-%H-%M-%S' if has_time else '%Y-%m-%d')
+            raise NotFoundError(message, url)
+
+        # If no index has been given, set it to the last build of the day.
+        self.show_matching_builds(parser.entries)
+        if build_index is None:
+            build_index = len(parser.entries) - 1
 
         return (parser.entries, build_index)
 
-
     @property
     def binary_regex(self):
         """Return the regex for the binary"""
@@ -357,7 +428,6 @@ class DailyScraper(Scraper):
                         'PLATFORM': self.platform_regex,
                         'EXT': self.extension}
 
-
     def build_filename(self, binary):
         """Return the proposed filename with extension for the binary"""
 
@@ -370,43 +440,46 @@ class DailyScraper(Scraper):
             timestamp = self.date.strftime('%Y-%m-%d')
 
         return '%(TIMESTAMP)s-%(BRANCH)s-%(NAME)s' % {
-                   'TIMESTAMP': timestamp,
-                   'BRANCH': self.branch,
-                   'NAME': binary}
-
+            'TIMESTAMP': timestamp,
+            'BRANCH': self.branch,
+            'NAME': binary}
 
     @property
     def monthly_build_list_regex(self):
-        """Return the regex for the folder which contains the builds of a month."""
+        """Return the regex for the folder containing builds of a month."""
 
         # Regex for possible builds for the given date
         return r'nightly/%(YEAR)s/%(MONTH)s/' % {
-                  'YEAR': self.date.year,
-                  'MONTH': str(self.date.month).zfill(2) }
-
+            'YEAR': self.date.year,
+            'MONTH': str(self.date.month).zfill(2)}
 
     @property
     def path_regex(self):
         """Return the regex for the path"""
 
         try:
-            return self.monthly_build_list_regex + self.builds[self.build_index]
+            path = ''.join([self.monthly_build_list_regex,
+                            self.builds[self.build_index]])
+            return path
         except:
-            raise NotFoundException("Specified sub folder cannot be found",
-                                    self.base_url + self.monthly_build_list_regex)
+            folder = ''.join([self.base_url, self.monthly_build_list_regex])
+            raise NotFoundError("Specified sub folder cannot be found",
+                                folder)
 
 
 class DirectScraper(Scraper):
     """Class to download a file from a specified URL"""
 
     def __init__(self, url, *args, **kwargs):
-        Scraper.__init__(self, *args, **kwargs)
-
         self.url = url
 
+        Scraper.__init__(self, *args, **kwargs)
+
     @property
     def target(self):
-        return urllib.splitquery(self.final_url)[0].rpartition('/')[-1]
+        target = urlparse(self.final_url)
+        filename = target.path.rpartition('/')[-1] or target.hostname
+        return os.path.join(self.directory, filename)
 
     @property
     def final_url(self):
@@ -432,7 +505,6 @@ class ReleaseScraper(Scraper):
         return regex[self.platform] % {'APP': self.application,
                                        'EXT': self.extension}
 
-
     @property
     def path_regex(self):
         """Return the regex for the path"""
@@ -442,7 +514,6 @@ class ReleaseScraper(Scraper):
                         'PLATFORM': self.platform_regex,
                         'VERSION': self.version}
 
-
     def build_filename(self, binary):
         """Return the proposed filename with extension for the binary"""
 
@@ -458,35 +529,44 @@ class ReleaseCandidateScraper(ReleaseScraper):
     """Class to download a release candidate build from the Mozilla server"""
 
     def __init__(self, build_number=None, no_unsigned=False, *args, **kwargs):
-        Scraper.__init__(self, *args, **kwargs)
-
-        # Internally we access builds via index
-        if build_number is not None:
-            self.builds = ['build%s' % build_number]
-            self.build_index = 0
-        else:
-            self.builds, self.build_index = self.get_build_info_for_version(self.version)
 
+        self.build_number = build_number
         self.no_unsigned = no_unsigned
         self.unsigned = False
 
+        Scraper.__init__(self, *args, **kwargs)
+
+    def get_build_info(self):
+        "Defines additional build information"
+
+        # Internally we access builds via index
+        if self.build_number is not None:
+            self.builds = ['build%s' % self.build_number]
+            self.build_index = 0
+        else:
+            self.builds, self.build_index = self.get_build_info_for_version(
+                self.version)
 
     def get_build_info_for_version(self, version, build_index=None):
         url = '/'.join([self.base_url, self.candidate_build_list_regex])
 
         print 'Retrieving list of candidate builds from %s' % url
-        parser = DirectoryParser(url)
+        parser = DirectoryParser(url, authentication=self.authentication,
+                                 timeout=self.timeout_network)
         if not parser.entries:
-            message = 'Folder for specific candidate builds at has not been found'
-            raise NotFoundException(message, url)
+            message = 'Folder for specific candidate builds at %s has not' \
+                'been found' % url
+            raise NotFoundError(message, url)
 
-        # If no index has been given, set it to the last build of the given version.
+        self.show_matching_builds(parser.entries)
+
+        # If no index has been given, set it to the last build of the given
+        # version.
         if build_index is None:
             build_index = len(parser.entries) - 1
 
         return (parser.entries, build_index)
 
-
     @property
     def candidate_build_list_regex(self):
         """Return the regex for the folder which contains the builds of
@@ -494,8 +574,7 @@ class ReleaseCandidateScraper(ReleaseScraper):
 
         # Regex for possible builds for the given date
         return r'nightly/%(VERSION)s-candidates/' % {
-                 'VERSION': self.version }
-
+            'VERSION': self.version}
 
     @property
     def path_regex(self):
@@ -508,11 +587,11 @@ class ReleaseCandidateScraper(ReleaseScraper):
                         'PLATFORM': self.platform_regex,
                         'UNSIGNED': "unsigned/" if self.unsigned else ""}
 
-
     def build_filename(self, binary):
         """Return the proposed filename with extension for the binary"""
 
-        template = '%(APP)s-%(VERSION)s-%(BUILD)s.%(LOCALE)s.%(PLATFORM)s.%(EXT)s'
+        template = '%(APP)s-%(VERSION)s-%(BUILD)s.%(LOCALE)s.' \
+                   '%(PLATFORM)s.%(EXT)s'
         return template % {'APP': self.application,
                            'VERSION': self.version,
                            'BUILD': self.builds[self.build_index],
@@ -520,14 +599,13 @@ class ReleaseCandidateScraper(ReleaseScraper):
                            'PLATFORM': self.platform,
                            'EXT': self.extension}
 
-
     def download(self):
         """Download the specified file"""
 
         try:
             # Try to download the signed candidate build
             Scraper.download(self)
-        except NotFoundException, e:
+        except NotFoundError, e:
             print str(e)
 
             # If the signed build cannot be downloaded and unsigned builds are
@@ -535,7 +613,8 @@ class ReleaseCandidateScraper(ReleaseScraper):
             if self.no_unsigned:
                 raise
             else:
-                print "Signed build has not been found. Falling back to unsigned build."
+                print "Signed build has not been found. Falling back to" \
+                      " unsigned build."
                 self.unsigned = True
                 Scraper.download(self)
 
@@ -551,42 +630,50 @@ class TinderboxScraper(Scraper):
 
     def __init__(self, branch='mozilla-central', build_number=None, date=None,
                  debug_build=False, *args, **kwargs):
-        Scraper.__init__(self, *args, **kwargs)
 
         self.branch = branch
+        self.build_number = build_number
         self.debug_build = debug_build
-        self.locale_build = self.locale != 'en-US'
-        self.timestamp = None
+        self.date = date
 
+        self.timestamp = None
         # Currently any time in RelEng is based on the Pacific time zone.
-        self.timezone = PacificTimezone();
+        self.timezone = PacificTimezone()
+
+        Scraper.__init__(self, *args, **kwargs)
+
+    def get_build_info(self):
+        "Defines additional build information"
 
         # Internally we access builds via index
-        if build_number is not None:
-            self.build_index = int(build_number) - 1
+        if self.build_number is not None:
+            self.build_index = int(self.build_number) - 1
         else:
             self.build_index = None
 
-        if date is not None:
+        if self.date is not None:
             try:
-                self.date = datetime.fromtimestamp(float(date), self.timezone)
-                self.timestamp = date
+                self.date = datetime.fromtimestamp(float(self.date),
+                                                   self.timezone)
+                self.timestamp = self.date
             except:
-                self.date = datetime.strptime(date, '%Y-%m-%d')
+                self.date = datetime.strptime(self.date, '%Y-%m-%d')
         else:
             self.date = None
 
+        self.locale_build = self.locale != 'en-US'
         # For localized builds we do not have to retrieve the list of builds
         # because only the last build is available
         if not self.locale_build:
-            self.builds, self.build_index = self.get_build_info(self.build_index)
-    
+            self.builds, self.build_index = self.get_build_info_for_index(
+                self.build_index)
+
             try:
                 self.timestamp = self.builds[self.build_index]
             except:
-                raise NotFoundException("Specified sub folder cannot be found",
-                                        self.base_url + self.monthly_build_list_regex)
-
+                raise NotFoundError("Specified sub folder cannot be found",
+                                    self.base_url +
+                                    self.monthly_build_list_regex)
 
     @property
     def binary_regex(self):
@@ -606,16 +693,14 @@ class TinderboxScraper(Scraper):
                         'LOCALE': self.locale,
                         'EXT': self.extension}
 
-
     def build_filename(self, binary):
         """Return the proposed filename with extension for the binary"""
 
         return '%(TIMESTAMP)s%(BRANCH)s%(DEBUG)s-%(NAME)s' % {
-                   'TIMESTAMP': self.timestamp + '-' if self.timestamp else '',
-                   'BRANCH': self.branch,
-                   'DEBUG': '-debug' if self.debug_build else '',
-                   'NAME': binary}
-
+            'TIMESTAMP': self.timestamp + '-' if self.timestamp else '',
+            'BRANCH': self.branch,
+            'DEBUG': '-debug' if self.debug_build else '',
+            'NAME': binary}
 
     @property
     def build_list_regex(self):
@@ -623,14 +708,16 @@ class TinderboxScraper(Scraper):
 
         regex = 'tinderbox-builds/%(BRANCH)s-%(PLATFORM)s%(L10N)s%(DEBUG)s'
 
-        return regex % {'BRANCH': self.branch,
-                        'PLATFORM': '' if self.locale_build else self.platform_regex,
-                        'L10N': 'l10n' if self.locale_build else '',
-                        'DEBUG': '-debug' if self.debug_build else ''}
-
+        return regex % {
+            'BRANCH': self.branch,
+            'PLATFORM': '' if self.locale_build else self.platform_regex,
+            'L10N': 'l10n' if self.locale_build else '',
+            'DEBUG': '-debug' if self.debug_build else ''}
 
     def date_matches(self, timestamp):
-        """Determines whether the timestamp date is equal to the argument date"""
+        """
+        Determines whether the timestamp date is equal to the argument date
+        """
 
         if self.date is None:
             return False
@@ -638,9 +725,8 @@ class TinderboxScraper(Scraper):
         timestamp = datetime.fromtimestamp(float(timestamp), self.timezone)
         if self.date.date() == timestamp.date():
             return True
-        
-        return False
 
+        return False
 
     @property
     def date_validation_regex(self):
@@ -648,21 +734,20 @@ class TinderboxScraper(Scraper):
 
         return r'^\d{4}-\d{1,2}-\d{1,2}$|^\d+$'
 
-
     def detect_platform(self):
         """Detect the current platform"""
 
         platform = Scraper.detect_platform(self)
 
-        # On OS X we have to special case the platform detection code and fallback
-        # to 64 bit builds for the en-US locale
-        if mozinfo.os == 'mac' and self.locale == 'en-US' and mozinfo.bits == 64:
+        # On OS X we have to special case the platform detection code and
+        # fallback to 64 bit builds for the en-US locale
+        if mozinfo.os == 'mac' and self.locale == 'en-US' and \
+                mozinfo.bits == 64:
             platform = "%s%d" % (mozinfo.os, mozinfo.bits)
 
         return platform
 
-
-    def get_build_info(self, build_index=None):
+    def get_build_info_for_index(self, build_index=None):
         url = '/'.join([self.base_url, self.build_list_regex])
 
         print 'Retrieving list of builds from %s' % url
@@ -670,7 +755,8 @@ class TinderboxScraper(Scraper):
         # If a timestamp is given, retrieve just that build
         regex = '^' + self.timestamp + '$' if self.timestamp else r'^\d+$'
 
-        parser = DirectoryParser(url)
+        parser = DirectoryParser(url, authentication=self.authentication,
+                                 timeout=self.timeout_network)
         parser.entries = parser.filter(regex)
 
         # If date is given, retrieve the subset of builds on that date
@@ -679,7 +765,9 @@ class TinderboxScraper(Scraper):
 
         if not parser.entries:
             message = 'No builds have been found'
-            raise NotFoundException(message, url)
+            raise NotFoundError(message, url)
+
+        self.show_matching_builds(parser.entries)
 
         # If no index has been given, set it to the last build of the day.
         if build_index is None:
@@ -687,7 +775,6 @@ class TinderboxScraper(Scraper):
 
         return (parser.entries, build_index)
 
-
     @property
     def path_regex(self):
         """Return the regex for the path"""
@@ -697,7 +784,6 @@ class TinderboxScraper(Scraper):
 
         return '/'.join([self.build_list_regex, self.builds[self.build_index]])
 
-
     @property
     def platform_regex(self):
         """Return the platform fragment of the URL"""
@@ -718,7 +804,7 @@ def cli():
     BUILD_TYPES = {'release': ReleaseScraper,
                    'candidate': ReleaseCandidateScraper,
                    'daily': DailyScraper,
-                   'tinderbox': TinderboxScraper }
+                   'tinderbox': TinderboxScraper}
 
     usage = 'usage: %prog [options]'
     parser = OptionParser(usage=usage, description=__doc__)
@@ -760,14 +846,14 @@ def cli():
     parser.add_option('--url',
                       dest='url',
                       metavar='URL',
-                      help='URL to download. Note: Reserved characters (such as &)\
-                           must be escaped or put in quotes otherwise CLI output\
-                           may be abnormal.')
+                      help='URL to download. Note: Reserved characters (such '
+                           'as &) must be escaped or put in quotes otherwise '
+                           'CLI output may be abnormal.')
     parser.add_option('--version', '-v',
                       dest='version',
                       metavar='VERSION',
-                      help='Version of the application to be used by release and\
-                            candidate builds, i.e. "3.6"')
+                      help='Version of the application to be used by release\
+                            and candidate builds, i.e. "3.6"')
     parser.add_option('--extension',
                       dest='extension',
                       metavar='EXTENSION',
@@ -790,18 +876,17 @@ def cli():
                            'the event of a failure, default: %default')
     parser.add_option('--retry-delay',
                       dest='retry_delay',
-                      default=10,
-                      type=int,
+                      default=10.,
+                      type=float,
                       metavar='RETRY_DELAY',
                       help='Amount of time (in seconds) to wait between retry '
                            'attempts, default: %default')
     parser.add_option('--timeout',
                       dest='timeout',
-                      default=180,
-                      type=int,
+                      type=float,
                       metavar='TIMEOUT',
-                      help='Amount of time (in seconds) until download times '
-                           'out, default: %default')
+                      help='Amount of time (in seconds) until a download times'
+                           ' out')
 
     # Option group for candidate builds
     group = OptionGroup(parser, "Candidate builds",
@@ -822,9 +907,9 @@ def cli():
                      metavar='BRANCH',
                      help='Name of the branch, default: "%default"')
     group.add_option('--build-id',
-                      dest='build_id',
-                      metavar='BUILD_ID',
-                      help='ID of the build to download')
+                     dest='build_id',
+                     metavar='BUILD_ID',
+                     help='ID of the build to download')
     group.add_option('--date',
                      dest='date',
                      metavar='DATE',
@@ -848,7 +933,8 @@ def cli():
     if not options.url \
        and not options.type in ['daily', 'tinderbox'] \
        and not options.version:
-        parser.error('The version of the application to download has not been specified.')
+        parser.error('The version of the application to download has not'
+                     ' been specified.')
 
     # Instantiate scraper and download the build
     scraper_keywords = {'application': options.application,
@@ -857,26 +943,22 @@ def cli():
                         'version': options.version,
                         'directory': options.directory,
                         'extension': options.extension,
-                        'authentication': {
-                            'username': options.username,
-                            'password': options.password},
+                        'authentication': (options.username, options.password),
                         'retry_attempts': options.retry_attempts,
                         'retry_delay': options.retry_delay,
                         'timeout': options.timeout}
-    scraper_options = {'candidate': {
-                           'build_number': options.build_number,
-                           'no_unsigned': options.no_unsigned},
-                       'daily': {
-                           'branch': options.branch,
-                           'build_number': options.build_number,
-                           'build_id': options.build_id,
-                           'date': options.date},
-                       'tinderbox': {
-                           'branch': options.branch,
-                           'build_number': options.build_number,
-                           'date': options.date,
-                           'debug_build': options.debug_build}
-                       }
+    scraper_options = {
+        'candidate': {'build_number': options.build_number,
+                      'no_unsigned': options.no_unsigned},
+        'daily': {'branch': options.branch,
+                  'build_number': options.build_number,
+                  'build_id': options.build_id,
+                  'date': options.date},
+        'tinderbox': {'branch': options.branch,
+                      'build_number': options.build_number,
+                      'date': options.date,
+                      'debug_build': options.debug_build}
+    }
 
     kwargs = scraper_keywords.copy()
     kwargs.update(scraper_options.get(options.type, {}))
